var retailers = [];
var current_retailer = {};
var coupons = [];
var products = [];
// Permanently disabled
var is_hidden = false;
// Temporarily disabled for 1 hour
var is_minimized = false;

const TEMP_DISABLED = 1;
const PERM_DISABLED = 2;

var iframe_url = "www.priceblink.com/iframe/";
var wishlist_url = "http://www.priceblink.com/webcpns/addwishlistitem.php";
var current_page_title = "";
// For coupon overlays and sent as part of universal scrape
var coupon_overlay_xpath = "";

// Begin the process where data contains supported retailers
self.port.on("init", function(data) {
  var url = document.URL;
  retailers = data;
  current_retailer = getRetailer(url);
  
  // Allow any page to be added to wishlist so this needs to happen before we check for supported retailers
  // We grab the page title and URL
  current_page_title = escape(getXPathContent("//title"));
  self.port.emit("setWishListURL", wishlist_url+"?title="+current_page_title+"&itemurl="+escape(url));  
  
  // Retailer exists so let's begin the process wth coupons
  if(current_retailer) {
    
    if(document.location.protocol == "https:")
      iframe_url = "https://" + iframe_url;
    else
      iframe_url = "http://" + iframe_url;
      
    // Call the addon script and tell it to make the coupon XHR, also retrieves the disabled status
    self.port.emit("getCoupons", current_retailer);
    
    // Listen for retailer minimize events
    var minimizeListener = function(evt) {
      // Disable coupons temporarily
      self.port.emit("disableCoupons", {id: current_retailer.id, dt: new Date().getTime()});
    }
    document.addEventListener("PB_Minimize_Event", minimizeListener, false, true);
    
    // Listen for retailer disable events
    var disableListener = function(evt) {
      // Disable coupons permanently
      self.port.emit("disableCoupons", {id: current_retailer.id});
    }
    document.addEventListener("PB_Disable_Event", disableListener, false, true);
    
    // Listen for toolbar restore event after being temporarily disabled
    var restoreListener = function(evt) {
      self.port.emit("restoreCoupons", current_retailer.id);
    }
    document.addEventListener("PB_Restore_Event", restoreListener, false, true);
  }
});

// Called from addon script when coupons are retrieved
self.port.on("couponsResponse", function(obj) {
  coupons = obj.coupons;
  coupon_overlay_xpath = obj.coupon_overlay_xpath;
  
  if(obj.disabled_status == TEMP_DISABLED)
    is_minimized = true;
  else if (obj.disabled_status == PERM_DISABLED)
    is_hidden = true;
  
  // Begin the product scrape
  beginScrape();
  
  if(document.URL.indexOf("www.google.com") != -1) {
    var g_interval = null;
    var url = document.URL;
    
    // This is for when the first search happens directly from a link or omnibox
    setTimeout(injectCoupons, 1000);
    
    // Listen for a URL change
    g_interval = setInterval(function() {
      // Only execute when url changes or previous injection has completed
      if(url.length != document.URL.length) {
        
        // Set the url to new one
        url = document.URL;
        
        // We introduce a delay in certain cases where instant results happen too fast
        setTimeout(beginInjection, 1000);
        
      } else {
        // URL hasn't changed
      }
      
    }, 500);
  }
  
});

function beginInjection() {
  // This means the iframe exists so we need to remove it
  if(document.getElementById("pb-div")) {
    document.body.removeChild(document.getElementById("pb-div"));
    var styles = document.getElementsByTagName('link');
    for(var i=0; i<styles.length; i++) {
      if(styles[i].href.indexOf("iframe/css/injected.css")) {
        styles[i].parentNode.removeChild(styles[i]);
        break;
      }
    }
  }

  injectCoupons();

  // Kick off the scrape
  beginScrape();
}

function injectCoupons() {
  if(coupon_overlay_xpath == "") return;
  
  var result = document.evaluate(coupon_overlay_xpath, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
  var urls = [];
  
  // No scrape results
  if(result.snapshotLength == 0) return;
  
  // Grab the links from the page
  for(var i=0; i < result.snapshotLength; i++) {
    var node = result.snapshotItem(i);
    var url = node.getAttribute("href");
    
    // This is necessary because of encoded url redirects
    url = url.split("//");
    url = "http://" + url[url.length-1];
    
    urls.push(url);
  }
  
  // Send URLs to background
  self.port.emit("getCouponsForMultipleURLs", urls);
  
}

// Receives message from background
self.port.on("getCouponsForMultipleURLsResponse", function(obj){
  var r = obj.coupons;
  var result = document.evaluate(coupon_overlay_xpath, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
  
  injectCSS("coupon_overlay.css");
  
  // Loop through the results again and find matches based on supported coupon retailers
  for(var i=0; i < result.snapshotLength; i++) {
    var node = result.snapshotItem(i);
    var url = node.getAttribute("href");
    url = url.split("//");
    url = "http://" + url[url.length-1];
    
    for(var j=0; j < r.length; j++) {
      if(doUrlsMatch(url,r[j].domain)) {
        var div = document.createElement("div");
        var span = document.createElement("span");
        span.setAttribute("class", "pb-coupon-overlay");
        span.appendChild(document.createTextNode(r[j].short_label));
        
        // This is a relative url
        if(node.getAttribute("href").indexOf("/") == 0) {
          div.setAttribute("data-href", r[j].link_prefix + escape("http://" + document.URL.split("/")[2] + node.getAttribute("href")));
          div.addEventListener("click", function(){location.href=this.getAttribute("data-href")});
        } else {
          div.setAttribute("data-href", r[j].link_prefix + escape(node.getAttribute("href")));
          div.addEventListener("click", function(){location.href=this.getAttribute("data-href")});
        }
                    
        div.setAttribute("class", "pb-coupon-div");
        div.setAttribute("title", r[j].label + "\n" + r[j].restrictions + "\n\nTo get this discount, click this button and then look for the coupon in the PriceBlink bar.");
        div.appendChild(span);

        // Put it right next to the hyperlink
        node.parentNode.appendChild(div);
      }
    }
  }
});

var doUrlsMatch = function(page_url, parser_url) {
  page_url = "/" + page_url.split("/")[2];
  
  var index = page_url.indexOf("/" + parser_url);
	
	if(index != -1) {
		return true;
	}
	
	index = page_url.indexOf("." + parser_url);
	
	if(index != -1) {
		return true;
	}

	return false;
}

// Called from addon script when products are retrieved
self.port.on("productsResponse", function(data) {
  // We made the call but we don't have any alternatives to offer. Need to make sure there are no coupons as well.
  if((data[0].retailers == undefined || data[0].retailers.length == 0) && (coupons[0].coupons.length == 0 || is_hidden)) 
    return;
  
  products = data;
  
  // Update the wishlist URL to overwrite the static one in cases we have a url from the server
  self.port.emit("setWishListURL", products[0].addtowishlist+"&itemurl="+escape(content.document.location.href));

  // Let's display the toolbar
  injectScript();
});

// Called after coupons are complete and we're looking for a universal scrape
self.port.on("universalScrape", function(data) {
    // Let's see if the user is on an order confirmation page
    var count = getXPathContent(data[0].order_confirmation_xpath);
    if(count > 0) {
      self.port.emit("checkForNotifications", current_retailer.id);
    }
});

var injectScript = function() {
  
  var script = document.createElement("script");
  script.type = "text/javascript";
  script.text = "window.addEventListener('message', PB_receiveMessage, false);function PB_receiveMessage(e){var pb_iframe=document.getElementById('pb-iframe');var pb_div=document.getElementById('pb-div');if(e.data=='maximize'){pb_iframe.style.height=pb_div.style.height='100%';}else if(e.data=='minimize'){pb_iframe.style.height=pb_div.style.height='36px';}else if(e.data=='close'){document.body.className=document.body.className.replace(' priceblink-body', '');PB_disable();var e = document.createEvent('Events');e.initEvent('PB_Minimize_Event', true, false);document.dispatchEvent(e);}else if(e.data=='restore'){var e = document.createEvent('Events');e.initEvent('PB_Restore_Event', true, false);document.dispatchEvent(e);document.location.href=document.location.href;}else if(e.data=='hide'){document.body.className=document.body.className.replace(' priceblink-body', '');pb_div.style.display='none';var e = document.createEvent('Events');e.initEvent('PB_Disable_Event', true, false);document.dispatchEvent(e);}}function PB_disable(){pb_iframe.src='" + iframe_url + "toolbar_minimized.html';document.getElementById('pb-div').style.width='25px'}";
  document.getElementsByTagName("head").item(0).appendChild(script);
  
  injectCSS("injected.css");

  // Don't shift the page if the toolbar is minimized
  if(!is_minimized || (products.length > 0 && products[0].retailers != undefined))
    document.body.className += " priceblink-body";

  injectIframe();
}

// Append the stylesheet to the document
var injectCSS = function(stylesheet) {
  
  // First check to make sure the CSS doesn't already exist
  var css_exists = false;
  
  // Loop through CSS
  var styles = document.getElementsByTagName('link');
  for(var i=0; i<styles.length; i++) {
    if(styles[i].href.indexOf("iframe/css/" + stylesheet) != -1) {
      css_exists = true;
      break;
    }
  }
  
  // CSS doesn't exist so create it
  if(!css_exists) {
    var css = document.createElement("link");
    css.type = "text/css";
    css.rel = "stylesheet";
    css.href = iframe_url + "css/" + stylesheet;  
    document.getElementsByTagName("head").item(0).appendChild(css);
  }
}

var injectIframe = function() {
  
  var iframe = document.createElement("iframe");
  var div = document.createElement("div");
  
  // In cases where the iframe is minimized but we're doing a products comparison we need to display the full toolbar
  if(is_minimized && products.length > 0 && products[0].retailers != undefined) {
    iframe.src = iframe_url + "toolbar.html";
    iframe.width = "100%";
    iframe.style.left = 0;
    div.style.width = "100%";
  // Display minimized iframe
  } else if(is_minimized) {
    iframe.src = iframe_url + "toolbar_minimized.html";
    iframe.width = "25px";
    div.style.width = "25px";
  // Display full iframe
  } else {
    iframe.src = iframe_url + "toolbar.html";
    iframe.width = "100%";
    iframe.style.left = 0;
    div.style.width = "100%";
  }
  
  div.id = "pb-div";
  div.style.height = "36px";
  div.style.cssFloat="right";
  div.style.position = "fixed";
  div.style.top = 0;
  div.style.right = 0;
  div.style.zIndex = "1000000";

  iframe.height = "36px";
  iframe.id = "pb-iframe";
  iframe.border = "none";
  iframe.frameBorder = "0";
  iframe.scrolling = "no";
  //iframe.style.position = "fixed";
  iframe.style.height = "36px";
  //iframe.style.top = 0;
  //iframe.style.right = 0;
  iframe.style.border = "none";
  //iframe.style.zIndex = "1000000";
  
  div.appendChild(iframe);
  document.body.insertBefore(div, document.body.firstChild);
  
  iframe.onload = function() {
    // If the toolbar is minimized and we're not on a products page don't do anything
    if(is_minimized && products.length == 0) return;
    
    // Escape single quotes and double escape double quotes
    var cstr = JSON.stringify(coupons).replace(/'/g,"\\'").replace(/\"/g,"\\\"");
    var script = document.createElement("script");
	  script.type = "text/javascript";
	  
	  // Display coupons only
	  if(products.length == 0) {
	    script.text = "var pb_iframe=document.getElementById('pb-iframe');pb_iframe.contentWindow.postMessage('" + cstr + "', '*');";
	  } else {
	    // Display coupons and products
	    var pstr = JSON.stringify(products).replace(/'/g,"\\'").replace(/\"/g,"\\\"");
	    script.text = "var pb_iframe=document.getElementById('pb-iframe');pb_iframe.contentWindow.postMessage('" + cstr + "|||" + pstr + "', '*');";
    }
    
	  var head = document.getElementsByTagName("head").item(0);
	  head.appendChild(script);
	  head.removeChild(script);
  }
}

// Check to make sure the current URL matches a supported retailer before proceeding
var getRetailer = function(url) {
  url = "/" + url.split("/")[2];

  if(url.indexOf(".dell.com") != -1)
    return getDell();

  for (var i=0; i<retailers.length; i++) {
    var index = url.indexOf("/" + retailers[i].url);
    
    if(index != -1) {
      return retailers[i];
    }
    
    index = url.indexOf("." + retailers[i].url);
    
    if(index != -1) {
      return retailers[i];
    }
  }
  return 0;
}

// In the case of dell we detect home vs business
var getDell = function() {
  // We need to watch the case of SEGMENT
  var xpath = "//META[@name='SEGMENT' or @name='Segment']/@content";
  var result = getXPathContent(xpath);
  
  if(result == 'dhs' || result == 'gen') {
    return getRetailerByName("Dell Home");
  } else if (result == 'bsd'){
    return getRetailerByName("Dell Business");
  } else {
    return 0;
  }
}

// Begin the scraping process
var beginScrape = function() {
  // If there's no title to scrape then let's stop and display coupons only
  if ((current_retailer.title == undefined || current_retailer.title == "") && coupons[0].coupons.length > 0 && !is_hidden) {
    injectScript();
    return;
  }

  var params = {};
  var title = escape(getXPathContent(current_retailer.title));
  params.title = title;
  // This is for the wish list button in cases where we use the url provided by the server
  current_page_title = title;
  debug("Title: " + title);
  
  // No title result so show coupons
  if(title == "" && coupons[0].coupons.length != 0 && !is_hidden) {
    injectScript();
    return;
  }
  
  var price = escape(getXPathContent(current_retailer.price));
  params.price = price;
  debug("Price: " + price);
  
  var model = escape(getXPathContent(current_retailer.model));
  params.model = model;
  debug("Model: " + model);
  
  var mpn = escape(getXPathContent(current_retailer.mpn));
  params.mpn = mpn;
  debug("MPN: " + mpn);
  
  var brand = escape(getXPathContent(current_retailer.brand));
  params.brand = brand;
  debug("Brand: " + brand);
  
  var upc = escape(getXPathContent(current_retailer.upc));
  params.upc = upc;
  debug("UPC: " + upc);
  
  var sku = escape(getXPathContent(current_retailer.sku));
  params.sku = sku;
  debug("SKU: " + sku);
  
  var isbn = escape(getXPathContent(current_retailer.isbn));
  params.isbn = isbn;
  debug("ISBN: " + isbn);
  
  var ship = escape(getXPathContent(current_retailer.ship));
  params.ship = ship;
  debug("Ship: " + ship);
  
  var rating = escape(getXPathContent(current_retailer.rating));
  params.rating = rating;
  debug("Rating: " + rating);
  
  var in_stock = escape(getXPathContent(current_retailer.in_stock));
  params.in_stock = in_stock;
  debug("In stock: " + in_stock);
  
  var c1 = escape(getXPathContent(current_retailer.c1));
  params.c1 = c1;
  debug("Custom 1: " + c1);
  
  var c2 = escape(getXPathContent(current_retailer.c2));
  params.c2 = c2;
  debug("Custom 2: " + c2);
  
  var c3 = escape(getXPathContent(current_retailer.c3));
  params.c3 = c3;
  debug("Custom 3: " + c3);
  
  var c4 = escape(getXPathContent(current_retailer.c4));
  params.c4 = c4;
  debug("Custom 4: " + c4);
  
  var c5 = escape(getXPathContent(current_retailer.c5));
  params.c5 = c5;
  debug("Custom 5: " + c5);
    
  params.rid = current_retailer.id;
  
  // Show product comparison
  if(title != '' && (sku != '' || mpn != '' || model != '' || upc != '' || isbn != ''))
    self.port.emit("getProducts", params);
  // A title was scraped but nothing else so let's show coupons
  // We've added logic so that if there are ONLY user coupons we still display them
  else if((coupons[0].coupons.length > 0 || (coupons[0].users_coupons != null && coupons[0].users_coupons.length > 0)) && !is_hidden)
    injectScript();
}

var debug = function(str) {
  //console.log(str);
}

// Handles XPath scrapes
var getXPathContent = function(xpath) {
  if(xpath == undefined || xpath == "") return "";
  
  try {
    var xpath = "normalize-space(" + xpath + ")";
    var doc = content.document;
    var result = doc.evaluate(xpath, doc, null, XPathResult.ANY_TYPE, null);
    return clean(result.stringValue);
  } catch (e) {
    return "";
  }
}

// Return a retailer object when passed a retailer name
var getRetailerByName = function(name) {
  for (var i=0; i<retailers.length;i++) {
    if(retailers[i].name == name) {
      return retailers[i];
    }
  }
}

// Begin utility functions to clean data
var clean = function(str) {
  return single_space(trim(str));
}

var single_space = function(str) {
  return str.replace(/^\s*|\s(?=\s)|\s*$/g, '');
}

var trim = function(str) {
  return str.replace(/^\s*|\s*$/g, '');
}

var strip = function(str) {
  return str.replace(/[^a-zA-Z 0-9]+/g,'');
}
// End utility functions