var page = require("page-mod");
var req = require("request").Request;
var self = require("self");
var tabs = require("tabs");
var timer = require("timers");
var data = self.data;
var storage = require("simple-storage").storage;
const {Cu} = require("chrome");

const NOT_DISABLED = 0;
const TEMP_DISABLED = 1;
const PERM_DISABLED = 2;

var PriceBlink = {};
PriceBlink.mode = 1; // 0 = DEV, 1 = PROD
PriceBlink.uid = "";
PriceBlink.ver = 3.5;
PriceBlink.browser = "ff";
PriceBlink.url = "http://tb.priceblink.com/";
PriceBlink.retailers = [];
PriceBlink.universal_scrapes = [];
PriceBlink.current_retailer_id = 0;
PriceBlink.disabled_status = NOT_DISABLED;
PriceBlink.current_tab = {};
PriceBlink.wishlist_url = "";

PriceBlink.wishlist_pref = "extensions.info@priceblink.com.wishlist";
PriceBlink.amazon_pref = "extensions.info@priceblink.com.amazon";
PriceBlink.ebay_pref = "extensions.info@priceblink.com.ebay";
PriceBlink.facebook_pref = "extensions.info@priceblink.com.facebook";
PriceBlink.gmail_pref = "extensions.info@priceblink.com.gmail";
PriceBlink.twitter_pref = "extensions.info@priceblink.com.twitter";

PriceBlink.tbb = {};
PriceBlink.amazon_tbb = {};
PriceBlink.ebay_tbb = {};
PriceBlink.facebook_tbb = {};
PriceBlink.gmail_tbb = {};
PriceBlink.twitter_tbb = {};

// Needed during testing
//storage.uid = null;

// If in dev mode
if (PriceBlink.mode == 0) {
  storage.uid = "test";
}

// Begin UID functions
PriceBlink.S4 = function() {
  return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
}

PriceBlink.guid = function () {
  return (PriceBlink.S4()+PriceBlink.S4()+"-"+PriceBlink.S4()+"-"+PriceBlink.S4()+"-"+PriceBlink.S4()+"-"+PriceBlink.S4()+PriceBlink.S4()+PriceBlink.S4());
}
// End UID functions

// First install so set UID and show install page
if (!storage.uid && PriceBlink.mode) {
  
  // Store the version
  storage.ver = PriceBlink.ver;
  
  var prefs = require("preferences-service");
  
  // There's a previous uid stored as a preference
  try {
    var uid = prefs.getCharPref("extensions.priceblink.uid");
    storage.uid = uid;
    
    // Delete the preference branch
    prefs.deleteBranch("extensions.priceblink");
  // There isn't a previous uid from an older version
  } catch(ex) {
    storage.uid = PriceBlink.guid();
    tabs.open(PriceBlink.url + "install?uid=" + storage.uid + "&browser=" + PriceBlink.browser + "&ver=" + PriceBlink.ver);
  }
}

// Detect existing version for the upgrade process
if(storage.ver) {
  
  var existing_version = storage.ver;
  existing_version = parseFloat(existing_version);
  
  // If this is an upgrade let's send them to the upgrade page
  if(existing_version < parseFloat(PriceBlink.ver)) {
    // Do nothing right now
  }
  
  // Store the current version
  storage.ver = PriceBlink.ver;
}

// Build params for requests
PriceBlink.url_params = "?uid=" + storage.uid + "&browser=" + PriceBlink.browser + "&ver=" + PriceBlink.ver;
PriceBlink.uid = storage.uid;

var getParser = function() {
  req({
    url: PriceBlink.url + "retailers_new_ff.php" + PriceBlink.url_params,
    onComplete: function (response) {
      PriceBlink.retailers = response.json;
    }
  }).get();
}

// Get the parser file on init
getParser();

// Let's grab the parser every 8 hours
var interval = timer.setInterval(function(){
  getParser();
}, 28800000);

// Universal scrape call
req({
  url: PriceBlink.url + "universal_scrapes.php" + PriceBlink.url_params,
  onComplete: function (response) {
    PriceBlink.universal_scrapes = response.json;
  }
}).get();

// Begin the scraping process
page.PageMod({
  include: ["*"],
  contentScriptWhen: 'ready',
  contentScriptFile: data.url("content.js"),
  attachTo: ["top"],
  onAttach: function(worker) {
    
    PriceBlink.current_tab = worker.tab;
    
    worker.port.emit("init", PriceBlink.retailers);
    
    // Get coupons and return them to content script
    worker.port.on("getCoupons", function(retailer) {
      var url = PriceBlink.url + "coupons" + PriceBlink.url_params + "&rid=" + retailer.id;
      
      PriceBlink.disabled_status = checkIfRetailerIsDisabled(retailer.id);
      PriceBlink.current_retailer_id = retailer.id;
      
      req({
        url: url,
        onComplete: function(response) {
          worker.port.emit("couponsResponse", {coupons: response.json, disabled_status: PriceBlink.disabled_status, coupon_overlay_xpath: PriceBlink.universal_scrapes[0].coupon_overlay_xpath});
          
          // There are coupons and the retailer is hidden so let's show the non-dimmed icon
          if(response.json[0].coupons.length && PriceBlink.disabled_status == PERM_DISABLED) {
            // Change icon to show coupons
            widget.contentURL = data.url("pb_logo_small.png");
            widget.tooltip = "Show coupons for this merchant - PriceBlink";
          } else {
            widget.contentURL = data.url("pb_logo_small_dim.png");
            widget.tooltip = "PriceBlink Add-on";
          }
          
          // Let's trigger the universal scrape after we get coupons
          worker.port.emit("universalScrape", PriceBlink.universal_scrapes);
        }
      }).get();
    });
    
    // Get products and return them to content script
    worker.port.on("getProducts", function(product) {
      var url = PriceBlink.url + "products" + PriceBlink.url_params + "&rid=" + product.rid;
      url += "&sku=" + product.sku;
      url += "&price=" + product.price;
      url += "&model=" + product.model;
      url += "&mpn=" + product.mpn;
      url += "&brand=" + product.brand;
      url += "&upc=" + product.upc;
      url += "&isbn=" + product.isbn;
      url += "&title=" + product.title;
      url += "&ship=" + product.ship;
      url += "&rating=" + product.rating;
      url += "&in_stock=" + product.in_stock;
      url += "&c1=" + product.c1;
      url += "&c2=" + product.c2;
      url += "&c3=" + product.c3;
      url += "&c4=" + product.c4;
      url += "&c5=" + product.c5;
      
      req({
        url: url,
        onComplete: function(response) {
          worker.port.emit("productsResponse", response.json);
        }
      }).get();
    });
    
    // Received when a user wants to disable coupons
    // We store an array of retailer objects with id and datetime of disabling
    // In the case of permanent disable the dt will be null
    // If the dt is not null the coupons will be temporarily disabled (1 hr)
    worker.port.on("disableCoupons", function(retailer) {
      
      if(storage.disabled_retailers == null) {
        storage.disabled_retailers = [];
      }
      
      var exists = checkRetailerIdExistsInStorage(retailer.id);
      
      if(!exists) {
        storage.disabled_retailers.push(retailer);
        // If dt is null then let's enable the addon icon immediately so a user can re-enable
        if(retailer.dt == null) {
          widget.contentURL = data.url("pb_logo_small.png");
          widget.tooltip = "Show coupons for this merchant - PriceBlink"
          PriceBlink.disabled_status = PERM_DISABLED;
        }
      }
      
    });
    
    // Restore coupons by removing retailer from simple storage
    worker.port.on("restoreCoupons", function(id){
      restoreCouponsForRetailer(id);
    });
    
    worker.port.on("setWishListURL", function(url){
      PriceBlink.wishlist_url = url + "&uid=" + PriceBlink.uid;
    });
    
    // This is called when we've got an order scrape in place and we want to set the cookie
    worker.port.on("checkForNotifications", function(id) {
      var url = PriceBlink.url + "universal_scrapes.php" + PriceBlink.url_params + "&rid=" + id;
      req({
        url: url,
        onComplete: function(response) {
          // Do nothing here. We just need to make the request and the proper cookie will either get set or not.
        }
      }).get();
    });
    
    // Sent from content script to get coupons for multiple urls
    worker.port.on("getCouponsForMultipleURLs", function(urls){
      var retailer_ids = getRetailerIdsForMultipleURLs(urls).toString();
      
      // There are no supported retailers so don't make the request
      if(retailer_ids.length == 0) return;
      
      var url = PriceBlink.url + "coupons_response_array.php" + PriceBlink.url_params + "&rid=" + PriceBlink.current_retailer_id + "&rids=" + retailer_ids;
      req({
        url: url,
        onComplete: function(response) {
          if(response.json == null) return;
          worker.port.emit("getCouponsForMultipleURLsResponse", response.json[0]);
        }
      }).get();
      
    });
    
}});

// For coupon injection functionality 
// return an array of supported retailers
var getRetailerIdsForMultipleURLs = function(urls) {
  var arr = [];
  for(var i=0; i<urls.length; i++) {
    var url = urls[i];
    var r = checkRetailerExistsInParser(url);
    if(r != null) {
      // If retailer already exists in array then don't add it
      var exists = false;
      for(var j=0; j<arr.length; j++) {
        if(arr[j].name == r.name) {
          exists = true;
          break;
        }
      }
      
      if(!exists)
        arr.push(r.id);
    }
  }
  return arr;
}

// Based on url let's see if the retailer exists in our parser
function checkRetailerExistsInParser(url) {
  url = "/" + url.split("/")[2];
  for (var i=0; i< PriceBlink.retailers.length; i++) {
		var index = url.indexOf("/" + PriceBlink.retailers[i].url);
		if(index != -1) {
			return PriceBlink.retailers[i];
		}
		index = url.indexOf("." + PriceBlink.retailers[i].url);
		if(index != -1) {
			return PriceBlink.retailers[i];
		}
	}
	return null;
}

var checkRetailerIdExistsInStorage = function(id) {
  var dr = storage.disabled_retailers;
  for(var i=0; i < dr.length; i++) {
    if(id == dr[i].id) {
     return true;
    }
  }
  return false;
}

// 0 means not disabled
// 1 means TEMP_DISABLED
// 2 means PERM_DISABLED
var checkIfRetailerIsDisabled = function(id) {
  var dr = storage.disabled_retailers;
  if(dr == null) return NOT_DISABLED;
  
  for(var i=0; i<dr.length; i++) {
    // First check that this retailer is in the disabled array
    if(id == dr[i].id) {
      // Let's check to see if there's a date/time which means it's temporarily disabled
      if(dr[i].dt != null) {
        // Check to see if temporary disabling has expired (1 hr)
        var offset = 1 * 60 * 60 * 1000;
        //var offset = 1 * 60 * 1 * 1000; // 1 min
        var nowDT = new Date().getTime();
        var disabledDT = parseInt(dr[i].dt);
        
        if(disabledDT + offset < nowDT) {
          restoreCouponsForRetailer(id);
          return NOT_DISABLED;
        } else {
          return TEMP_DISABLED;
        }
      // The retailer is permanently disabled
      } else {
        return PERM_DISABLED;
      }
    }
  }
  return NOT_DISABLED;
}

// Restore coupons after a retailer has been disabled temporarily
var restoreCouponsForRetailer = function(id) {
  var dr = storage.disabled_retailers;
  for(var i=0; i<dr.length; i++) {
    if(id == dr[i].id) {
      storage.disabled_retailers.splice(i, 1);
      return;
    }
  }
}

var widget = require("widget").Widget({
  id: "priceblink-widget",
  label: "PriceBlink",
  tooltip: "PriceBlink Add-on",
  contentURL: data.url("pb_logo_small_dim.png"),
});

// When the widget is clicked we want to unhide the retailer
widget.on("click", function(){
  try {
    if(PriceBlink.disabled_status == PERM_DISABLED) {
      restoreCouponsForRetailer(PriceBlink.current_retailer_id);
      // Reload the tab so coupons will display
      PriceBlink.current_tab.url = PriceBlink.current_tab.url;
    }
  } catch (ex) {}
});


// Uninstall cleanup
var listener = {
  // Can't seem to get this method to work
  onUninstalling: function(addon) {
    if(addon.id == "info@priceblink.com") {
      var uid = null;
      var ver = null;
      // Clear the storage
      storage.uid = uid;
      storage.ver = ver;
    }
  },
  
  onUninstalled: function(addon) {
    if(addon.id == "info@priceblink.com") {
      tabs.open({
        url: PriceBlink.url + "uninstall?uid=" + PriceBlink.uid,
        inNewWindow: true
      });
    }
  }
}

// Add the uninstall listener
try {
  var AddonManager = Cu.import("resource://gre/modules/AddonManager.jsm").AddonManager;
  AddonManager.addAddonListener(listener);
} catch (ex) {console.log("exception: " + ex);}


// Display a navigation toolbar button for universal wishlist
var createWishlistButton = function() {
  PriceBlink.tbb = require("toolbarbutton").ToolbarButton({
    id: "pb-wishlist",
    label: "",
    image: data.url("pb_logo_small.png"),
    tooltip: "Add this page to your PriceBlink wish list",

    onCommand: function () {
      tabs.open(PriceBlink.wishlist_url);
      inNewWindow: true;
    }
  });
  
  PriceBlink.tbb.moveTo({
    toolbarID: "nav-bar",
    forceMove: true
  });
}

// Preference to toggle toolbar buttons in navigation bar
var prefListener = function(name) {
  if(name == "wishlist")
    toggleWishlistButton();
}

require("simple-prefs").on("wishlist", prefListener);

// Shows/hides wishlist button in navigation bar
var toggleWishlistButton = function() {
  var display = require("preferences-service").get(PriceBlink.wishlist_pref);
  
  // Enable wishlist option by default on initial install
  if(display == undefined) {
    require("preferences-service").set(PriceBlink.wishlist_pref, true);
    display = true;
  }
  
  // Show the wishlist button
  if(display) {
    createWishlistButton();
  }
  // Hide the wishlist button
  else {
    PriceBlink.tbb.destroy();
  }
}

// Toggle buttons on startup
toggleWishlistButton();

var createToolbarButton = function(button_name, button_id, button_image, button_tooltip, button_url) {
  PriceBlink[button_name] = require("toolbarbutton").ToolbarButton({
    id: button_id,
    label: "",
    image: data.url(button_image),
    tooltip: button_tooltip,

    onCommand: function () {
      tabs.activeTab.url = button_url;
    }
  });
  
  PriceBlink[button_name].moveTo({
    toolbarID: "nav-bar",
    forceMove: true
  });
}